﻿Type=Class
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private mCategories As List
	Private mSelected As Category

	Private callback As Object
	Private eventName As String
End Sub

'Initialize collection
Public Sub Initialize()
	callback=Null
	eventName=""
	
	mCategories.Initialize
	Fill
	mSelected=Null
	If mCategories.Size>0 Then mSelected=mCategories.Get(0)
End Sub

Public Sub SetCallback(receiver As Object,event As String)
	callback=receiver
	eventName=event
End Sub

'Set selected Category
Public Sub ChangeSelection(item As Category)
	mSelected=mCategories.Get(mCategories.IndexOf(item))
	If Not(callback=Null) Then
		CallSub(callback, eventName & "_SelectionChange")
	End If
End Sub

#Region Properties
'Return all Categories as List
Sub getAll As List
	Return mCategories
End Sub

'Return selected Category
Sub getSelected As Category
	Return mSelected
End Sub
#End Region

'Create a new Category
Public Sub CreateNew() As Category
	Private c As Category
	c.Initialize("","")
	Return c
End Sub

'Add Category to List
Public Sub Add(item As Category)
	If Not(item.IsNew) Then
		mCategories.Add(item)
	Else
		Log("Before check: " & item.IsNew)
		If CheckExist(item) Then
			Log("After check: " & item.IsNew)
			Save(item)
		Else
			If Save(item) Then Insert(item)
		End If
	End If
End Sub

'Update Category
Public Sub Update(item As Category)
	Save(item)
End Sub

'Remove Category from List
Public Sub Remove(item As Category)
	Private index As Int
	If mCategories.IndexOf(mSelected)=mCategories.IndexOf(item) Then
		index=mCategories.IndexOf(item)
		If index>0 Then index=index-1
	Else
		index=mCategories.IndexOf(mSelected)
	End If
	item.Delete
	If Save(item) Then
		mCategories.RemoveAt(mCategories.IndexOf(item))
		If mCategories.Size>0 Then ChangeSelection(mCategories.Get(index))
	End If
End Sub

'Return List Lenght
Public Sub Lenght As Int
	Return mCategories.Size
End Sub

#Region Private Sub
Private Sub Fill
	Private db As Database
	db.Initialize
	mCategories.clear
	Private db_category As Cursor
	Try
		db_category=db.GetData("SELECT Description,Notes FROM Categories ORDER BY lower(Description);",Null)
		For i=0 To db_category.RowCount-1
			db_category.Position=i
			Private c As Category
			c.Initialize(db_category.GetString("Description"),db_category.GetString("Notes"))
			mCategories.Add(c)
		Next
		db_category.Close
	Catch
		Log(LastException)
	End Try
	db.Close
End Sub

Private Sub CheckExist(item As Category) As Boolean
	Private found As Category
	found=Null
	For Each c As Category In mCategories
		If c.Description.ToLowerCase.CompareTo(item.Description.ToLowerCase)=0 Then found=c
	Next

	If Not(found=Null) Then
		If Msgbox2("Category already exists. Overwrite existing category?",item.Description,"Yes","","No",Null)=DialogResponse.POSITIVE Then
			found.Notes=item.Notes
			'Save(found)
			Log("Before assign: " & item.IsNew)
			item=found
			Log("After assign: " & item.IsNew)
			Return True
		End If
	End If
	Return False
End Sub

Private Sub Insert(item As Category)
	Select Case mCategories.Size
		Case 0
			'First item into list
			mCategories.Add(item)
			Exit
		Case 1
			'List has just 1 item
			Private first As Category=mCategories.Get(0)
			If (item.Description.ToLowerCase.CompareTo(first.Description.ToLowerCase))<0 Then 
				mCategories.InsertAt(0,item)
			Else
				mCategories.Add(item)
			End If
			Exit
		Case Else
			'List has more items
			Private first As Category=mCategories.Get(0)
			Private last As Category=mCategories.Get(mCategories.Size-1)
			
			Select Case True
			Case (item.Description.ToLowerCase.CompareTo(first.Description.ToLowerCase))<0
				'Insert item on top of the list
				mCategories.InsertAt(0,item)
				Exit
			Case (item.Description.ToLowerCase.CompareTo(last.Description.ToLowerCase))>0
				'insert item on bottom of the list
				mCategories.Add(item)
				Exit
			Case Else
				'Insert item into the list
				For i=0 To mCategories.Size-2 'use -2 because last element has not following items
					Private previous As Category=mCategories.Get(i)
					Private following As Category=mCategories.Get(i+1)
					If previous.Description.ToLowerCase.CompareTo(item.Description.ToLowerCase)<0 And item.Description.ToLowerCase.CompareTo(following.Description.ToLowerCase)<0 Then
						mCategories.InsertAt(i+1,item)
					End If
				Next
			End Select
	End Select
End Sub

'Save Category
Private Sub Save(item As Category) As Boolean	
	Return item.Save
End Sub
#End Region