﻿Type=Class
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private mProducts As List
	Private p_category As Category
	Private p_brand As Brand
	Private mSelected As Product

	Private callback As Object
	Private eventName As String
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(category As Category,brand As Brand)
	p_category=category
	p_brand=brand
	callback=Null
	eventName=""
	
	mProducts.Initialize
	Fill
	mSelected=Null
	If mProducts.Size>0 Then mSelected=mProducts.Get(0)
End Sub

Public Sub SetCallback(receiver As Object,event As String)
	callback=receiver
	eventName=event
End Sub

'Set selected Product
Public Sub ChangeSelection(item As Product)
	mSelected=mProducts.Get(mProducts.IndexOf(item))
	If Not(callback=Null) Then
		CallSub(callback, eventName & "_SelectionChange")
	End If
End Sub

#Region Properties
'Return all Products as List
Sub getAll As List
	Return mProducts
End Sub

'Return selected Product
Sub getSelected As Product
	Return mSelected
End Sub
#End Region

'Create a new Product
Public Sub CreateNew() As Product
	Private p As Product
	p.Initialize(p_category.Description,"","","","")
	Return p
End Sub

'Add Product to List
Public Sub Add(item As Product)
	If item.IsNew Then 
		If item.Model="" Or item.Brand="" Then 
			item=Null
			Return
		End If
		
		Private found As Product
		found=Existing(item)
		If Not(found=Null) Then
			If Msgbox2("Product already exists. Update existing product?",item.Text,"Yes","","No",Null) Then DialogResponse.POSITIVE Then
				found.BrandCode=item.BrandCode
				found.Notes=item.Notes
				item=found
			Else
				item=Null
				Return
			End If
		Else
			Insert(item)
		End If
		If Save(item) Then ChangeSelection(item)
	Else
		mProducts.Add(item)
	End If
End Sub

'Update Product
Public Sub Update(item As Product)
	Save(item)
End Sub

'Remove Product from List
Public Sub Remove(item As Product)
	Private index As Int
	If mProducts.IndexOf(mSelected)=mProducts.IndexOf(item) Then
		index=mProducts.IndexOf(item)
		If index>0 Then index=index-1
	Else
		index=mProducts.IndexOf(mSelected)
	End If
	item.Delete
	If Save(item) Then
		mProducts.RemoveAt(mProducts.IndexOf(item))
		If mProducts.Size>0 Then ChangeSelection(mProducts.Get(index))
	End If
End Sub

'Return List Lenght
Public Sub Lenght As Int
	Return mProducts.Size
End Sub

#Region Private Sub
Private Sub Filter As String()
	Private data() As String
	
	If p_category=Null Then
		data(0)=""
		data(1)=""
	Else
		data(0)=p_category.Description
		data(1)=p_category.Description
	End If

	If p_brand=Null Then
		data(2)=""
		data(3)=""
	Else
		data(2)=p_brand.Description
		data(3)=p_brand.Description
	End If
		
	Return Filter
End Sub
 
Private Sub Fill
	Private db As Database
	db.Initialize
	mProducts.clear
	Private db_products As Cursor
	Try
		db_products=db.GetData("SELECT Category,Brand,Model,Brandcode,Notes FROM Products WHERE (Category=? OR ?="") AND (Brand=? OR ?="") ORDER BY lower(Brand),lower(Model);",Filter)
	Catch
		Log(LastException)
	End Try
	For i=0 To db_products.RowCount-1
		db_products.Position=i
		Private p As Product
		p.Initialize(db_products.GetString("Category"),db_products.GetString("Brand"),db_products.GetString("Model"),db_products.GetString("Brandcode"),db_products.GetString("Notes"))
		mProducts.Add(p)
	Next
	db_products.Close
	db.Close
End Sub

Private Sub Insert(item As Product)
	Select Case mProducts.Size
		Case 0
			'First item into list
			mProducts.Add(item)
			Exit
		Case 1
			'List has just 1 item
			Private first As Product=mProducts.Get(0)
			If (item.Text.ToLowerCase.CompareTo(first.Text.ToLowerCase))<0 Then 
				mProducts.InsertAt(0,item)
			Else
				mProducts.Add(item)
			End If
			Exit
		Case Else
			'List has more items
			Private first As Product=mProducts.Get(0)
			Private last As Product=mProducts.Get(mProducts.Size-1)
			
			Select Case True
			Case (item.Text.ToLowerCase.CompareTo(first.Text.ToLowerCase))<0
				'Insert item on top of the list
				mProducts.InsertAt(0,item)
				Exit
			Case (item.Text.ToLowerCase.CompareTo(last.Text.ToLowerCase))>0
				'insert item on bottom of the list
				mProducts.Add(item)
				Exit
			Case Else
				'Insert item into the list
				For i=0 To mProducts.Size-2 'use -2 because last element has not following items
					Private previous As Product=mProducts.Get(i)
					Private following As Product=mProducts.Get(i+1)
					If previous.Text.ToLowerCase.CompareTo(item.Text.ToLowerCase)<0 And item.Text.ToLowerCase.CompareTo(following.Text.ToLowerCase)<0 Then
						mProducts.InsertAt(i+1,item)
					End If
				Next
			End Select
	End Select
End Sub

'Save Product
Private Sub Save(item As Product) As Boolean
	Private result As Boolean

	result=item.Save(Null)
	If Not(result) Then Msgbox("Error on modifying the db.","Error")	
	
	Return result
End Sub

'Checks if Product exists
Private Sub Existing(product As Product) As Product
	Private found As Product
	found=Null
	For Each p As Product In mProducts
		If (p.Category.ToLowerCase.CompareTo(product.Category.ToLowerCase)=0) And (p.Brand.ToLowerCase.CompareTo(product.Brand.ToLowerCase)=0) And (p.Model.ToLowerCase.CompareTo(product.Model.ToLowerCase)=0) Then found=p
	Next
	Return found
End Sub
#End Region