﻿Type=Class
Version=5.2
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module
Sub Class_Globals
	Private mShops As List
	Private mSelected As Shop

	Private callback As Object
	Private eventName As String
End Sub

'Initialize collection
Public Sub Initialize()
	callback=Null
	eventName=""
	
	mShops.Initialize
	Fill
	mSelected=Null
	If mShops.Size>0 Then mSelected=mShops.Get(0)
End Sub

Public Sub SetCallback(receiver As Object,event As String)
	callback=receiver
	eventName=event
End Sub

'Set selected Shop
Public Sub ChangeSelection(item As Shop)
	mSelected=mShops.Get(mShops.IndexOf(item))
	If Not(callback=Null) Then
		CallSub(callback, eventName & "_SelectionChange")
	End If
End Sub

#Region Properties
'Return all Shops as List
Sub getAll As List
	Return mShops
End Sub

'Return selected Shop
Sub getSelected As Shop
	Return mSelected
End Sub
#End Region

'Create a new Shop
Public Sub CreateNew() As Shop
	Private s As Shop
	s.Initialize("","")
	Return s
End Sub

'Add Shop to List
Public Sub Add(item As Shop)
	If item.IsNew Then 
		If item.Description="" Then 
			item=Null
			Return
		End If
		
		Private found As Shop
		found=Existing(item)
		If Not(found=Null) Then
			If Msgbox2("Shop already exists. Update existing Shop?",item.Description,"Yes","","No",Null) Then DialogResponse.POSITIVE Then
				found.Notes=item.Notes
				item=found
			Else
				item=Null
				Return
			End If
		Else
			Insert(item)
		End If
		If Save(item) Then ChangeSelection(item)
	Else
		mShops.Add(item)
	End If
End Sub

'Update Shop
Public Sub Update(item As Shop)
	Save(item)
End Sub

'Remove Shop from List
Public Sub Remove(item As Shop)
	Private index As Int
	If mShops.IndexOf(mSelected)=mShops.IndexOf(item) Then
		index=mShops.IndexOf(item)
		If index>0 Then index=index-1
	Else
		index=mShops.IndexOf(mSelected)
	End If
	item.Delete
	If Save(item) Then
		mShops.RemoveAt(mShops.IndexOf(item))
		If mShops.Size>0 Then ChangeSelection(mShops.Get(index))
	End If
End Sub

'Return List Lenght
Public Sub Lenght As Int
	Return mShops.Size
End Sub

#Region Private Sub
Private Sub Fill
	Private db As Database
	db.Initialize
	mShops.clear
	Private db_shops As Cursor
	Try
		db_shops=db.GetData("SELECT Description,Notes FROM Shops ORDER BY lower(Description);",Null)
	Catch
		Log(LastException)
	End Try
	For i=0 To db_shops.RowCount-1
		db_shops.Position=i
		Private s As Shop
		s.Initialize(db_shops.GetString("Description"),db_shops.GetString("Notes"))
		mShops.Add(s)
	Next
	db_shops.Close
End Sub

Private Sub Insert(item As Shop)
	Select Case mShops.Size
		Case 0
			'First item into list
			mShops.Add(item)
			Exit
		Case 1
			'List has just 1 item
			Private first As Shop=mShops.Get(0)
			If (item.Description.ToLowerCase.CompareTo(first.Description.ToLowerCase))<0 Then 
				mShops.InsertAt(0,item)
			Else
				mShops.Add(item)
			End If
			Exit
		Case Else
			'List has more items
			Private first As Shop=mShops.Get(0)
			Private last As Shop=mShops.Get(mShops.Size-1)
			
			Select Case True
			Case (item.Description.ToLowerCase.CompareTo(first.Description.ToLowerCase))<0
				'Insert item on top of the list
				mShops.InsertAt(0,item)
				Exit
			Case (item.Description.ToLowerCase.CompareTo(last.Description.ToLowerCase))>0
				'insert item on bottom of the list
				mShops.Add(item)
				Exit
			Case Else
				'Insert item into the list
				For i=0 To mShops.Size-2 'use -2 because last element has not following items
					Private previous As Shop=mShops.Get(i)
					Private following As Shop=mShops.Get(i+1)
					If previous.Description.ToLowerCase.CompareTo(item.Description.ToLowerCase)<0 And item.Description.ToLowerCase.CompareTo(following.Description.ToLowerCase)<0 Then
						mShops.InsertAt(i+1,item)
					End If
				Next
			End Select
	End Select
End Sub

'Save Shop
Private Sub Save(item As Shop) As Boolean
	Private result As Boolean

	result=item.Save(Null)	
	If Not(result) Then Msgbox("Error on modifying the db.","Error")
	
	Return result
End Sub

'Checks if Shop exists
Private Sub Existing(shop As Shop) As Shop
	Private found As Shop
	found=Null
	For Each s As Shop In mShops
		If s.Description.ToLowerCase.CompareTo(shop.Description.ToLowerCase)=0 Then found=s
	Next
	Return found
End Sub
#End Region